#
#  AppController.rb
#  Matchback Magic
#
#  Created by Gregory Clarke on 10/7/09.
#  Copyright (c) 2009 Intelligent Assistance, Inc. All rights reserved.
#
require 'osx/cocoa'
OSX.require_framework 'QTKit'
require 'matchback_magic_algorithm.bundle'

class AppController < OSX::NSObject
	
	ib_outlets :mainWindow, :deactivateMenuItem, :saveMenuItem
	ib_outlets :protectView, :folderIcon, :folderName, :chooseFolderButton, :protectXmlIcon, :protectXmlFileName, :chooseProtectXmlButton, :spinner, :protectButton, :saveToALEButton1
	ib_outlets :simplifyView, :simplifyFolderIcon, :simplifyFolderName, :simplifyFolderButton, :audioTracksTableView, :changeMovsButton, :simplifySpinner
	ib_outlets :conformView, :chooseConformXmlButton, :conformXmlIcon, :conformXmlFileName, :spreadsheetTableView, :saveToExcelButton, :saveToALEButton2, :saveToFCPButton
	ib_outlets :progressWindow, :progressView, :progressIndicator, :progressMessage
	ib_outlets :regoView, :deactivateAreaView, :webView, :problemMessage, :closeRegoButton

	# String constants for the toolbar.
	TOOLBAR_IDENTIFIER = 'My Window Toolbar Identifier'
	TOOLBAR_FIRST_IDENTIFIER = 'First Toolbar Item Identifier'
	TOOLBAR_SECOND_IDENTIFIER = 'Second Toolbar Item Identifier'
	TOOLBAR_THIRD_IDENTIFIER = 'Third Toolbar Item Identifier'

	ib_action :showProtect do |sender|
		showProtectView()
	end

	ib_action :showSimplify do |sender|
		showSimplifyView()
	end

	ib_action :showConform do |sender|
		showConformView()
	end

	def awakeFromNib
		puts "awakeFromNib"
		setupToolbar
		# will use CoreAnimation for the window changes:
		@mainWindow.contentView.wantsLayer = true
	end

	### rego methods ###
	
	def applicationWillFinishLaunching(sender) # applicationWillFinishLaunching: is sent before application:openFile: (so it can deal with a file dropped onto the app icon)
		puts "applicationWillFinishLaunching"
		check_for_expired_beta()
		@current_view = @protectView
		if @this_rego[0]["active"]
			showProtectView()
		elsif @this_rego[0]["mac_problem"]==true
			check_for_rego(@this_rego)
		else
			@deactivateMenuItem.setTitle("Activate...")
			displayDemoSheet()
		end
	end
	
	def displayDemoSheet
		alert = OSX::NSAlert.alloc.init
		alert.setMessageText("#{SKU} has not been activated for this computer.")
		alert.setAlertStyle(OSX::NSInformationalAlertStyle)
		alert.setInformativeText("Would you like to activate #{SKU}, or use it in free mode?")
		alert.addButtonWithTitle("Activate")
		alert.addButtonWithTitle("Buy")
		alert.addButtonWithTitle("Free")
		alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, "didEndDemoSheet:returnCode:contextInfo:", nil)
	end

	def didEndDemoSheet_returnCode_contextInfo(sheet, buttonClicked, info)
		sheet.window.orderOut(nil)
		if buttonClicked==OSX::NSAlertSecondButtonReturn
			system('open', 'http://www.AssistedEditing.com/MatchbackMagic/buy')
		end
		if buttonClicked==OSX::NSAlertThirdButtonReturn
			showProtectView()
		else
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC)
		end
	end

	def showDeactivation
		if @deactivateMenuItem.title=="Activate..."
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			newFrame = self.newFrameForNewContentView(@deactivateAreaView) # 500 x 200
			@mainWindow.setMinSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setMaxSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setContentView(@deactivateAreaView)
			showActivationView(newFrame, WEB_APP + "destroy/" + @this_rego[0]["number"].to_s)
		end
		@closeRegoButton.setTitle("Cancel")
	end
	ib_action :showDeactivation # Matchback Magic for FCP > Activate or Deactivate
	
	def showActivationView(frame, url)
		puts "showActivationView #{url}"
		@mainWindow.setFrame_display frame, true
		@mainWindow.setContentView(@regoView)
		@mainWindow.setTitle("Activation")
		# notification
		center = OSX::NSNotificationCenter.defaultCenter
		center.addObserver_selector_name_object(self, :webViewProgressFinished, "WebProgressFinishedNotification", @webView) # name is WebProgressFinishedNotification NOT WebViewProgressFinishedNotification
		@webView.mainFrame.loadRequest( OSX::NSURLRequest.requestWithURL( OSX::NSURL.URLWithString(url) ) )
	end
	
	def webViewProgressFinished(notification)
		puts notification.name
		if notification.name=="WebProgressFinishedNotification"
			if @webView.mainFrame.dataSource.nil? # in case page doesn't load...
				@webView.mainFrame.stopLoading
				@problemMessage.setHidden(false)
				@webView.setHidden(true)
			else
				get_web_app_response(@webView.mainFrame.dataSource.response.URL)
			end
		end
	end 
	
	def closeActivationView
		if @closeRegoButton.title=="Continue" || @closeRegoButton.title=="Cancel"
			showProtectView()
		else
			OSX::NSApplication.sharedApplication.terminate(self)
		end
	end
	ib_action :closeActivationView # on click of Registration window's Continue button
	
	### end rego methods ###
	
	def showProtectView
		puts "showProtectView"
		@toolbar.selectedItemIdentifier = @toolbar.items[0].itemIdentifier # select the first toolbar item
		width, height = 500, 265
		setupView(@protectView, "Matchback Magic for FCP", width, height)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setMaxSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setShowsResizeIndicator(false)
		@folderIcon.setHidden(true)
		@folderIcon.setToolTip("")
		@folderName.setStringValue("")
		@protectXmlIcon.setHidden(true)
		@protectXmlIcon.setToolTip("")
		@protectXmlFileName.setStringValue("")
		@protectButton.setEnabled(false)
	end

	def showSimplifyView
		puts "showSimplifyView"
		@toolbar.selectedItemIdentifier = @toolbar.items[1].itemIdentifier
		width, height = 500, 285
		setupView(@simplifyView, "Matchback Magic for FCP", width, height)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setMaxSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setShowsResizeIndicator(false)
		@simplifyFolderIcon.setHidden(true)
		@simplifyFolderIcon.setToolTip("")
		@simplifyFolderName.setStringValue("")
		@audio_tracks_table_checkboxes = []
		@audioTracksTableView.reloadData
	end

	def showConformView
		puts "showConformView"
		@toolbar.selectedItemIdentifier = @toolbar.items[2].itemIdentifier
		width, height = 600, 400
		setupView(@conformView, "Matchback Magic for FCP", width, height)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setMaxSize(OSX::NSSize.new(10000, 10000))
		@mainWindow.setShowsResizeIndicator(true)
		@conformXmlIcon.setHidden(true)
		@conformXmlIcon.setToolTip("")
		@conformXmlFileName.setStringValue("")
	end

	def setupView(view, title, width, height)
		@mainWindow.setTitle(title)
		oldFrame = @mainWindow.frame
		newFrame = self.newFrameForNewContentView(view)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setFrame_display_animate(newFrame, true, true)
		@current_view = view
		@mainWindow.setContentView(view)
	end

	def newFrameForNewContentView(view)
		newFrameRect = @mainWindow.frameRectForContentRect(view.frame)
		oldFrameRect = @mainWindow.frame
		newSize = newFrameRect.size
		oldSize = oldFrameRect.size
		frame = @mainWindow.frame
		frame.size = newSize
		frame.origin.y = frame.origin.y - (newSize.height - oldSize.height)
		frame
	end

	def showHelp(sender)
		openPDF("Matchback Magic for FCP - Help.pdf")
	end
	ib_action :showHelp # Help > Matchback Magic for FCP Help
	
	def openPDF(filename)
		path_to_file = __FILE__.sub("AppController.rb",filename)
		system('open', path_to_file)
	end

	def visitWebsite(sender)
		system('open', 'http://www.AssistedEditing.com/')
	end
	ib_action :visitWebsite # Help > Visit www.AssistedEditing.com

	def sendFeedback(sender)
		OSX::JRFeedbackController.showFeedback
	end
	ib_action :sendFeedback # Matchback Magic for FCP > Provide Feedback

	### Protect methods ###

	def chooseProtectFolder(sender)
		@protect_folder_path = nil
		@folderIcon.setHidden(true)
		@folderIcon.setToolTip("")
		@folderName.setStringValue("")
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setCanChooseDirectories(true)
		oPanel.setCanChooseFiles(false)
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, nil, @mainWindow, self, 'didEndFolderOpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseProtectFolder
	
	def didEndFolderOpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			@protect_folder_path = oPanel.filenames.objectAtIndex(0)
			@folderIcon.setHidden(false)
			@folderIcon.setToolTip(@protect_folder_path.to_s)
			@folderName.setStringValue(File.basename(@protect_folder_path))
			@folderIcon.setNeedsDisplay(true)
			@folderName.setNeedsDisplay(true)
		end
		if @protect_folder_path.nil? || @protect_xml_path.nil?
			@protectButton.setEnabled(false)
		else
			@protectButton.setEnabled(true)
		end
	end
	
	def chooseProtectXmlFile(sender)
		@protect_xml_path = nil
		@protectXmlIcon.setHidden(true)
		@protectXmlIcon.setToolTip("")
		@protectXmlFileName.setStringValue("")
		filetypes = ["xml"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndProtectXmlOpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseProtectXmlFile

	def didEndProtectXmlOpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			@saveToALEButton1.setEnabled(false)
			oPanel.orderOut(self)
			@protect_xml_path = oPanel.filenames.objectAtIndex(0)
			@protectXmlIcon.setHidden(false)
			@protectXmlIcon.setToolTip(@protect_xml_path.to_s)
			@protectXmlFileName.setStringValue(File.basename(@protect_xml_path))
			@protectXmlIcon.setNeedsDisplay(true)
			@protectXmlFileName.setNeedsDisplay(true)
		end
		if @protect_folder_path.nil? || @protect_xml_path.nil?
			@protectButton.setEnabled(false)
		else
			@protectButton.setEnabled(true)
		end
	end

	def saveProtectXml(sender)
		begin # FIXME
			successful_injection = make_protection_xml()
			alert = OSX::NSAlert.alloc.init
			if successful_injection==0
				alert.setMessageText("No files were found in the folder matching clip names in the XML")
				alert.setAlertStyle(OSX::NSCriticalAlertStyle)
				alert.setInformativeText("Please choose another folder of media or another XML file exported from Final Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
			else
				alert.setMessageText("Success!")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				alert.setInformativeText("Matchback information was added to #{successful_injection} media files. \nSwitching to Final Cut Pro.")
				alert.addButtonWithTitle("Continue")
				alert.runModal
				open_it = OSX::NSWorkspace.sharedWorkspace.openFile_withApplication_andDeactivate(TEMP_XML_FILE, "Final Cut Pro", true) # do switch to FCP
				puts "did the file open? #{open_it}"
			end
		rescue
			didEndProgressSheet_returnCode_contextInfo(@progressWindow, -1, nil)
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("There was a problem sending this metadata to Final Cut Pro")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			puts "RESCUE!!! Problem with make_protection_xml()"
			alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
			alert.addButtonWithTitle("OK")
			alert.runModal
		end
	end
	ib_action :saveProtectXml
	
	### Simplify methods ###
	
	def chooseSimplifyFolder(sender)
		@simplifyFolderIcon.setHidden(true)
		@simplifyFolderIcon.setToolTip("")
		@simplifyFolderName.setStringValue("")
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setCanChooseDirectories(true)
		oPanel.setCanChooseFiles(false)
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, nil, @mainWindow, self, 'didEndFolder2OpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseSimplifyFolder
	
	def didEndFolder2OpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			@simplify_folder_path = oPanel.filenames.objectAtIndex(0)
			@simplifyFolderIcon.setHidden(false)
			@simplifyFolderIcon.setToolTip(@simplify_folder_path.to_s)
			@simplifyFolderName.setStringValue(File.basename(@simplify_folder_path))
			@simplifyFolderIcon.setNeedsDisplay(true)
			@simplifyFolderName.setNeedsDisplay(true)
		end
		if @simplifyFolderName.stringValue==""
			@changeMovsButton.setEnabled(false)
		else
			@changeMovsButton.setEnabled(true)
		end
		found_first_movie = false
		@audio_tracks_table_checkboxes = Array.new
		Dir.new(@simplify_folder_path).entries.each do |filename| # array of filenames
			unless found_first_movie==true #|| filename[0,0]=="."
				if File.file?(filename) || File.extname(filename)==".mov" || File.extname(filename)==".rev" || File.extname(filename)==".dv" || File.extname(filename)==""
					puts "#{filename} is a movie? found_first_movie = #{found_first_movie}"
					qt_movie, error = OSX::QTMovie.movieWithFile_error(@simplify_folder_path + "/" + filename)
					# Check to see if the first movie opened successfully.
					if qt_movie == nil or error != nil 
						puts "  could not load #{filename}" 
					else
						found_first_movie = true
						audio_tracks_array = qt_movie.tracksOfMediaType("soun")
						puts "  This movie has #{audio_tracks_array.length} audio tracks"
						audio_tracks_array.each do |t|
							@audio_tracks_table_checkboxes << {"format" => t.attributeForKey("QTTrackFormatSummaryAttribute").to_s, "state" => 1}
						end
						puts @audio_tracks_table_checkboxes.inspect
					end
				end
			end
		end
		@audioTracksTableView.reloadData
		puts "@audioTracksTableView #{@audioTracksTableView.inspect}"
		if @audio_tracks_table_checkboxes.length > 0
			@audioTracksTableView.deselectRow(0)
		end
	end
	
	def saveNewMovs(sender)
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setCanChooseDirectories(true)
		oPanel.setCanCreateDirectories(true)
		oPanel.setCanChooseFiles(false)
		oPanel.setAllowsMultipleSelection(false)
		oPanel.setPrompt("Save to folder")
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, nil, @mainWindow, self, 'didEndFolder3OpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :saveNewMovs
	
	def didEndFolder3OpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			@simplifySpinner.startAnimation(self)
			@simplify_new_folder_path = oPanel.filenames.objectAtIndex(0)
			begin
				Dir.new(@simplify_folder_path).entries.each do |filename| # array of filenames
					if File.extname(filename)==".mov" || File.extname(filename)==""
						save_new_mov(filename) if File.file?(@simplify_folder_path + "/" + filename)
					end
				end
			rescue
			end
			@simplifySpinner.stopAnimation(self)
		end
	end
	
	def save_new_mov(filename)
		qt_movie, error = OSX::QTMovie.movieWithFile_error(@simplify_folder_path + "/" + filename)
		if qt_movie == nil or error != nil 
			puts "  could not load #{filename}" 
		else
			qt_movie.setAttribute_forKey(true, OSX::QTMovieEditableAttribute)
			audio_tracks_array = qt_movie.tracksOfMediaType("soun")
			puts "  This movie has #{audio_tracks_array.length} audio tracks"
			@audio_tracks_table_checkboxes.each_with_index do |checkbox, i|
				if checkbox["state"]==0 # the checkbox is off
					qt_movie.removeTrack(audio_tracks_array[i])
				end
			end
			new_mov_path = @simplify_new_folder_path + "/" + filename
			qt_movie.writeToFile_withAttributes(new_mov_path, {"QTMovieFlatten" => 1})
		end
	end
	
	### Conform methods ###
	
	def chooseConformXmlFile(sender)
		@nokogiri_xml = nil
		@nokogiri_xml_clone = nil
		@clips_in_spreadsheet = nil
		@spreadsheetTableView.reloadData
		@saveMenuItem.setEnabled(false)
		@saveToExcelButton.setEnabled(false)
		@saveToALEButton2.setEnabled(false)
		@saveToFCPButton.setEnabled(false)
		@conformXmlIcon.setHidden(true)
		@conformXmlIcon.setToolTip("")
		@conformXmlFileName.setStringValue("")
		filetypes = ["xml"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndConformXmlOpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseConformXmlFile

	def didEndConformXmlOpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			file = oPanel.filenames.objectAtIndex(0)
			loadConformTableView(self, file)
		end
	end

	def loadConformTableView(sender, path)
		OSX::NSDocumentController.sharedDocumentController().noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath(path) ) # add this file to the Recent menu
		# fileManager = OSX::NSFileManager.defaultManager
		chosen_file = path.to_s
		chosen_file_extension = File.extname(chosen_file)
		@conformXmlIcon.setHidden(false)
		@conformXmlIcon.setToolTip(path.to_s)
		@conformXmlFileName.setStringValue(File.basename(chosen_file))
		@conformXmlIcon.setNeedsDisplay(true)
		@conformXmlFileName.setNeedsDisplay(true)
		begin # FIXME
			@progressIndicator.setIndeterminate(true)
			@progressIndicator.setUsesThreadedAnimation(true)
			@progressIndicator.startAnimation(self)
			displayProgressSheet()
			read_metadata_from_xml(chosen_file)
			@spreadsheetTableView.tableColumns.each_with_index do |tableColumn, i|
				# tableColumn.headerCell.setStringValue(@column_headings[i])
				tableColumn.setHeaderToolTip(@column_headings[i])
			end
			@spreadsheetTableView.reloadData
			@spreadsheetTableView.deselectAll(self)
			# @spreadsheetTableView.deselectRow(0)
			if @clips_in_spreadsheet.length == 0
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was no Matchback information in this XML file.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
				chooseConformXmlFile(nil)
			end
		rescue
			didEndProgressSheet_returnCode_contextInfo(@progressWindow, -1, nil)
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("There was a problem with this XML file.")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			puts "RESCUE!!! Problem with read_metadata_from_xml(chosen_file)"
			alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
			alert.addButtonWithTitle("OK")
			alert.runModal
			chooseConformXmlFile(nil)
		end
	end

	def displayProgressSheet
		OSX::NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@progressWindow, @mainWindow, self, 'didEndProgressSheet:returnCode:contextInfo:', nil)
		@progressView.displayIfNeeded
	end

	def didEndProgressSheet_returnCode_contextInfo(sheet, code, info)
		puts "didEndProgressSheet_returnCode_contextInfo code #{code}"
		sheet.orderOut(nil) # or self?
		@progressMessage.setStringValue("")
		@progressIndicator.stopAnimation(self)
		OSX::NSApp.endSheet(sheet)
	end

	def saveExcel
		sPanel = OSX::NSSavePanel.savePanel
		sPanel.setTitle("Save the Excel spreadsheet")
		sPanel.setRequiredFileType("xls")
		sPanel.setExtensionHidden(false)
		sPanel.setCanSelectHiddenExtension(true)
		sPanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, @mainWindow, self, 'didEndSaveConformExcel:returnCode:contextInfo:', nil)
	end
	ib_action :saveExcel
	
	def saveALE
		sPanel = OSX::NSSavePanel.savePanel
		sPanel.setTitle("Save the ALE")
		sPanel.setRequiredFileType("txt")
		sPanel.setExtensionHidden(false)
		sPanel.setCanSelectHiddenExtension(true)
		sPanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, @mainWindow, self, 'didEndSaveConformALE:returnCode:contextInfo:', nil)
	end
	ib_action :saveALE
	
	# save temporary XML file and open into FCP (no need for an open sheet)
	def saveConformXML
		@progressIndicator.setIndeterminate(true)
		@progressIndicator.setUsesThreadedAnimation(true)
		@progressIndicator.startAnimation(self)
		# displayProgressSheet()
		begin # FIXME
			reconform_fcp_sequence()
		rescue
			didEndProgressSheet_returnCode_contextInfo(@progressWindow, -1, nil)
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("There was a problem conforming this Final Cut Pro sequence.")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			puts "RESCUE!!! Problem with reconform_fcp_sequence()"
			alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
			alert.addButtonWithTitle("OK")
			alert.runModal
		end
	end
	ib_action :saveConformXML
	
	def didEndSaveConformExcel_returnCode_contextInfo(sPanel, buttonClicked, info)
		@spinner.startAnimation(self)
		########## MAKE AND SAVE EXCEL ##########
		if buttonClicked == OSX::NSOKButton
			sPanel.orderOut(self)
			if sPanel.isExtensionHidden?
				extension_hidden = 1
			else
				extension_hidden = 0
			end
			puts "is extension hidden?", sPanel.isExtensionHidden
			excel_file = sPanel.filenames.objectAtIndex(0).to_s
			begin # FIXME
				save_to_excel(excel_file)
				OSX::NSWorkspace.sharedWorkspace.openFile(excel_file)
			rescue
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was a problem saving an Excel spreadsheet of this metadata.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				puts "RESCUE!!! Problem with save_to_excel(excel_file)"
				alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
			end
		end
		@spinner.stopAnimation(self)
	end
	
	def didEndSaveConformALE_returnCode_contextInfo(sPanel, buttonClicked, info)
		@spinner.startAnimation(self)
		########## MAKE AND SAVE ALE ##########
		if buttonClicked == OSX::NSOKButton
			sPanel.orderOut(self)
			if sPanel.isExtensionHidden?
				extension_hidden = 1
			else
				extension_hidden = 0
			end
			puts "is extension hidden?", sPanel.isExtensionHidden
			ale_file = sPanel.filenames.objectAtIndex(0).to_s
			begin # FIXME
				save_to_ale(ale_file)
				OSX::NSWorkspace.sharedWorkspace.openFile(ale_file)
			rescue
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was a problem saving an ALE file of this metadata.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				puts "RESCUE!!! Problem with save_to_ale(ale_file)"
				alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
			end
		end
		@spinner.stopAnimation(self)
	end
	
	def applicationShouldTerminateAfterLastWindowClosed(sender)
		true
	end

	### Toolbar methods ###
	
	def setupToolbar
		# Create a new toolbar instance, and attach it to our document window.
		@toolbar ||= OSX::NSToolbar.alloc.initWithIdentifier(TOOLBAR_IDENTIFIER)
		# Set up toolbar properties: disallow customization, give a default display mode and don't remember state in user defaults.
		@toolbar.setAllowsUserCustomization(false)
		@toolbar.setAutosavesConfiguration(true)
		@toolbar.setDisplayMode(OSX::NSToolbarDisplayModeIconAndLabel)
		# We are the delegate.
		@toolbar.setDelegate(self)
		# Attach the toolbar to the window.
		@mainWindow.setToolbar(@toolbar)
	end

	def toolbarDefaultItemIdentifiers(toolbar)
		[ TOOLBAR_FIRST_IDENTIFIER, TOOLBAR_SECOND_IDENTIFIER, TOOLBAR_THIRD_IDENTIFIER ]
	end

	def toolbarAllowedItemIdentifiers(toolbar)
		[ TOOLBAR_FIRST_IDENTIFIER, TOOLBAR_SECOND_IDENTIFIER, TOOLBAR_THIRD_IDENTIFIER ]
	end
	
	def toolbarSelectableItemIdentifiers(toolbar)
		[ TOOLBAR_FIRST_IDENTIFIER, TOOLBAR_SECOND_IDENTIFIER, TOOLBAR_THIRD_IDENTIFIER ]
	end
	
	# don't need validateToolbarItem if all items will be shown at all times
	
	def toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar(toolbar, identifier, flag)
		label, tooltip, image, action = 
		case identifier.to_s
		when TOOLBAR_FIRST_IDENTIFIER
			[ "Protect", "Protect", "Protect", 'showProtect:' ]
		when TOOLBAR_SECOND_IDENTIFIER
			[ "Simplify", "Simplify", "Simplify", 'showSimplify:']
		when TOOLBAR_THIRD_IDENTIFIER
			[ "Conform", "Conform", "Conform", 'showConform:' ]
		else
			return nil
		end
		toolbarItem = OSX::NSToolbarItem.alloc.initWithItemIdentifier(identifier)
		toolbarItem.setLabel(label)
		toolbarItem.setPaletteLabel(label)
		toolbarItem.setToolTip(tooltip)
		toolbarItem.setImage(OSX::NSImage.imageNamed(image))
		toolbarItem.setTarget(self)
		toolbarItem.setAction(action)
		return toolbarItem
	end

	### TableView methods ###

	def numberOfRowsInTableView(tableView)
		if tableView.tag==1 # audio tracks
			if @audio_tracks_table_checkboxes.nil?
				return 0
			else
				return @audio_tracks_table_checkboxes.length
			end
		elsif tableView.tag==2 # conform spreadsheet
			if @clips_in_spreadsheet.nil?
				return 0
			else
				return @clips_in_spreadsheet.length
			end
		end
	end

	def tableView_objectValueForTableColumn_row(tableView, tableColumn, rowIndex)
		if tableView.tag==1 # audio tracks
			tableColumn.dataCellForRow(rowIndex).setTitle(@audio_tracks_table_checkboxes[rowIndex]["format"])
			return @audio_tracks_table_checkboxes[rowIndex]["state"]
		elsif tableView.tag==2 #conform spreadsheet
			@column_headings.each_with_index do |column, i|
				# unless tableColumn.nil?
				if tableColumn.headerCell.stringValue==column
					object = @clips_in_spreadsheet[rowIndex]
					return object[column]
				end
			end
		end
	end
	
	def tableView_setObjectValue_forTableColumn_row(tableView, anObject, tableColumn, rowIndex)
		if tableView.tag==1 # audio tracks
			tableView.deselectRow(rowIndex)
			@audio_tracks_table_checkboxes[rowIndex]["state"] = anObject.to_i
			puts @audio_tracks_table_checkboxes.inspect
		end
	end

end
